//
//  JSSLearnView.m
//  Pintoresco
//
//  Created by jsainz on 18/12/13.
//  Copyright (c) 2013 Javier Sainz. All rights reserved.
//

#define NSLog(FORMAT, ...) printf("%s\n", [[NSString stringWithFormat:FORMAT, ##__VA_ARGS__] UTF8String]);
#define ONLYNUMBERS @"1234567"

#import "JSSAppDelegate.h"
#import "JSSLearnView.h"
#import "JSSItemView.h"

static NSString *CellIdentifier = @"Cell";

@interface JSSLearnView ()

@property (strong, nonatomic) NSIndexPath *indexPathToBeDeleted;

@end

@implementation JSSLearnView

@synthesize tableView;
@synthesize indexPathToBeDeleted;
@synthesize itemFile;

@synthesize learnInfoUser;
@synthesize learnUser;
@synthesize identifier;
@synthesize summary;

#pragma mark - Life cycle

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = @"DATOS";
        self.view.backgroundColor = [UIColor colorWithRed:1 green:0.871 blue:0.678 alpha:1]; //FFDEAD
    }
    defaults = [[NSUserDefaults alloc]init];
    return self;
}

-(NSString *)tableView:(UITableView *)tableView
    titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath {
    return @"BORRAR";
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.leftBarButtonItem = self.editButtonItem;
    self.editButtonItem.title = NSLocalizedString(@"Borrar Datos", @"Edit");
    
    tableView.backgroundColor = [UIColor colorWithRed:1 green:0.871 blue:0.678 alpha:1]; //FFDEAD
    
//    self.navigationController.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(insertNewObject:)];
    
    
    UIBarButtonItem *printButton = [[UIBarButtonItem alloc] initWithTitle:@"Guardar Datos" style:UIBarButtonItemStylePlain target:self action:@selector(insertNewObject:)];
    self.navigationItem.rightBarButtonItem = printButton;
    
 
    for (NSInteger i=0; i<identifier.count; i++) {
        NSLog(@"Learn: %@", [identifier objectAtIndex:i]);
    }
    for (NSInteger i=0; i<summary.count; i++) {
        NSLog(@"Learn: %@", [summary objectAtIndex:i]);
    }
    
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
    [super setEditing:editing animated:animated];
    if (editing) {
        self.editButtonItem.title = NSLocalizedString(@"Cancelar", @"Done");
        [self endAlert:@"PARA BORRAR DATOS"];
    }
    else {
        self.editButtonItem.title = NSLocalizedString(@"Borrar Datos", @"Edit");
        
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - Fetched Results Controller Delegate

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView endUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath {
    // UITableView *table = self.tableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)
    sectionInfo atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type {
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView beginUpdates];
}

#pragma mark - Private Methods

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    NSManagedObject *object = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    NSMutableString *oString = [[NSMutableString alloc]init];
    
    [oString appendFormat:@"%@ ", [[object valueForKey:@"date"] description]];
    [oString appendFormat:@"%@ ", [[object valueForKey:@"identity"] description]];
    [oString appendFormat:@"%@ ", [[object valueForKey:@"name"] description]];
    [oString appendFormat:@"%@ ", [[object valueForKey:@"surname"] description]];
    
//    cell.textLabel.text = [[object valueForKey:@"date"] description];
    cell.textLabel.text = oString;
}

- (void)insertNewObject:(id)sender {
    JSSAppDelegate *appDelegate = (JSSAppDelegate*)[[UIApplication sharedApplication]delegate];
    
    // Entity for table name: (Experimental) Data
    NSEntityDescription *entity = [NSEntityDescription
                                   insertNewObjectForEntityForName:@"Data"
                                   inManagedObjectContext:appDelegate.managedObjectContext];
    
    defaults = [NSUserDefaults standardUserDefaults];
    NSString *userName = [defaults stringForKey:@"KEYUSERNAME"];
    NSString *surname = [defaults stringForKey:@"KEYSURNAME"];
    if (surname.length > 0) {
        NSString *name = [defaults stringForKey:@"KEYNAME"];
        NSString *surname = [defaults stringForKey:@"KEYSURNAME"];
        [entity setValue:name forKey:@"name"];
        [entity setValue:surname forKey:@"surname"];
        //        NSNumber *idCard = [defaults objectForKey:@"KEYIDCARD"];
        //        id media = [NSNumber numberWithInt:[idCard integerValue]];
        //        [entity setValue:media forKey:@"identity"];
    }
    else if (userName.length > 0) {
        [entity setValue:userName forKey:@"name"];
        [entity setValue:@"BBDD" forKey:@"surname"];
        //        id media = [NSNumber numberWithInt:00000000];
        //        [entity setValue:media forKey:@"identity"];
    }
 
    NSDate *timeStamp = [NSDate date];
    [entity setValue:timeStamp forKey:@"date"];

    NSMutableArray *taskArray = [NSMutableArray array];
    NSMutableString *oString = [[NSMutableString alloc]init];
    NSString *iString = [[NSMutableString alloc]init];
    JSSStimulus *oArray = [[JSSStimulus alloc]init];
    NSLog(@"Items #: %d", itemFile.count);
    
    for (NSInteger i = 0; i < itemFile.count; i++) {
        oArray = [itemFile objectAtIndex:i];

        iString = [NSString stringWithFormat:@"\r"];
        [oString appendFormat:@"%@ ", iString];
        
        iString = [NSString stringWithFormat:@"%02d", oArray.atIndex];
        [oString appendFormat:@"Item: %@ ", iString];
        
        iString = [NSString stringWithFormat:@"%hhd", oArray.category];
        [oString appendFormat:@"Cat: %@ ", iString];
        
        [oString appendFormat:@"Estimulo: %@ ", oArray.pictureCode];
        
        iString = [NSString stringWithFormat:@"%.6f", oArray.latency];
        [oString appendFormat:@"Latencia: %@ ", iString];
        
        iString = [NSString stringWithFormat:@"%d", oArray.error];
        [oString appendFormat:@"Error: %@", iString];
            // NSLog(@"%@", oString);
        [taskArray addObject:oString];
    }
    if ((surname > 0) || (userName > 0)) {
        [entity setValue:taskArray forKey:@"task"];
    
        NSError *error;
        BOOL isSaved = [appDelegate.managedObjectContext save:&error];
        NSLog(@"Datos guardados: %hhd", isSaved);
    }
}

#pragma mark - Properties

- (void)writeFile:(NSMutableArray*)m title:(NSString*)fileName {
    NSMutableString *oString = [[NSMutableString alloc]init];
    
    // NSString* filePath = @"/Users/jsainz/Desktop/";
    NSString *filePath = @"/Users/Load/";
    [oString appendFormat:@"%@.txt", fileName];
    NSString* fileAtPath = [filePath stringByAppendingPathComponent:oString];
    
    if (![[NSFileManager defaultManager] fileExistsAtPath:fileAtPath])
        [[NSFileManager defaultManager] createFileAtPath:fileAtPath contents:nil attributes:nil];
    
    oString = [[NSMutableString alloc]init];
    for (NSString *var in m) {
        [oString appendFormat:@"%@ \n", var];
        [[oString dataUsingEncoding:NSUTF8StringEncoding] writeToFile:fileAtPath atomically:NO];
    }
}

- (NSFetchedResultsController *)fetchedResultsController {
    JSSAppDelegate *appDelegate = (JSSAppDelegate*)[[UIApplication sharedApplication]delegate];
    self.managedObjectContext = appDelegate.managedObjectContext;
    
    if (_fetchedResultsController != nil)
        return _fetchedResultsController;
    
//    NSArray *keys = [[[NSUserDefaults standardUserDefaults] dictionaryRepresentation] allKeys];
//    for(NSString* key in keys){
//        
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYRULE"],key);
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYCATEGORY"],key);
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYPICTURE"],key);
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYSERIES"],key);
//
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYIDENTITY"],key);
//        NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:@"KEYSUBJECT"],key);
//        // NSLog(@"value: %@ forKey: %@",[[NSUserDefaults standardUserDefaults] valueForKey:key],key);
//    }
    
    // Create entity object for table: Data
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Data"
                                              inManagedObjectContext: appDelegate.managedObjectContext];
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc]init];
    [fetchRequest setEntity:entity];
    
    // Get all rows in a mutable array
    NSMutableArray *array = [[appDelegate.managedObjectContext
                              executeFetchRequest:fetchRequest error:nil]mutableCopy];
    
    NSLog(@"Número de archivos: %d", array.count);
//    NSMutableString *oString = [[NSMutableString alloc]init];
    NSMutableArray *data = [NSMutableArray array];

    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"HH:mm:ss Z"];
    
    NSString *fileName = [[NSString alloc]init];
    for(NSManagedObject *object in array) {
        NSLog(@"Fecha: %@", [object valueForKey:@"date"]);
        NSLog(@"Nombre: %@", [object valueForKey:@"name"]);
        NSLog(@"Apellidos %@", [object valueForKey:@"surname"]);
        
//        NSLog(@"Sujeto: %@", [object valueForKey:@"target"]);
//        [fetchRequest setReturnsDistinctResults:YES];
//        NSString *targetVar = [object valueForKey:@"target"];
//        [oString appendFormat:@"%@ ", targetVar];
//        NSLog(@"%@", oString);
        
        NSLog(@"%@", [object valueForKey:@"task"]);
        
//        [fetchRequest setReturnsDistinctResults:YES];
//        NSString *taskVar = [object valueForKey:@"task"];
//        [oString appendFormat:@"%@ ", taskVar];
//        NSLog(@"%@", oString);

        fileName = [[object valueForKey:@"date"] description];
        [data addObject:[object valueForKey:@"date"]];
//        [data addObject:[object valueForKey:@"identity"]];
//        [data addObject:[object valueForKey:@"name"]];
        [data addObject:[object valueForKey:@"surname"]];
//        [data addObject:[object valueForKey:@"target"]];
        [data addObject:[object valueForKey:@"task"]];
        [data addObject:@"\n--------------------------------\n"];
    }
    
    [self writeFile:data title:fileName];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:NO];
    NSArray *sortDescriptors = @[sortDescriptor];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    _fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:[self managedObjectContext] sectionNameKeyPath:nil cacheName:nil];
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error]) {
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    abort();
	}
    return _fetchedResultsController;
}

#pragma mark - Table view data source

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell
                                         forRowAtIndexPath:(NSIndexPath *)indexPath {
    cell.backgroundColor = [UIColor colorWithRed:1 green:0.871 blue:0.678 alpha:1];
}

// Determines the number of sections within this table view
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [[self.fetchedResultsController sections] count];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    return NO;
}

// Determines the number of rows for the argument section number
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController sections][section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil)
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
                                            forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];
        [context deleteObject:[self.fetchedResultsController objectAtIndexPath:indexPath]];
        
        NSError *error = nil;
        if (![context save:&error]) {
            NSLog(@"Error no identificado %@, %@", error, [error userInfo]);
            abort();
        }
    }
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath
                                                  toIndexPath:(NSIndexPath *)toIndexPath {
}

// Call the cell touch handler and pass in the cell associated
- (void)tableView:(UITableView *)tView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    if (cell == nil)
        cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    
    indexPathToBeDeleted = indexPath;
    NSLog(@"Borrar fila %d", indexPath.row + 1);
    [self buttonAlertPressed];
}

#pragma mark - Alert view management

- (void)endAlert:(NSString*)message {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:message message:nil delegate:nil cancelButtonTitle:@"ESCOGE UNA FILA" otherButtonTitles:nil];
    [alert show];
    [self performSelector:@selector(test:) withObject:alert afterDelay:3];
}

-(void)test:(UIAlertView*)alarm{
    [alarm dismissWithClickedButtonIndex:-1 animated:YES];
}

-(IBAction)buttonAlertPressed {
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle: @"Información"
                                                   message: @"Confirmar borrado"
                                                  delegate: self
                                         cancelButtonTitle: @"Cancelar"
                                         otherButtonTitles: @"Aceptar",nil];
    [alert show];
}

-(void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex == 0)
        ; // No action is to be taken
    else {
        NSLog(@"Fila borrada: %d", indexPathToBeDeleted.row + 1);
        [self removeItem:indexPathToBeDeleted];
    }
}

- (void)removeItem: (NSIndexPath *)indexPath {
    JSSAppDelegate *appDelegate = (JSSAppDelegate*)[[UIApplication sharedApplication]delegate];
    
    NSFetchRequest * allRecords = [[NSFetchRequest alloc] init];
    [allRecords setEntity:[NSEntityDescription entityForName:@"Data" inManagedObjectContext:appDelegate.managedObjectContext]];
    
    NSError * error = nil;
    NSArray * records = [appDelegate.managedObjectContext executeFetchRequest:allRecords error:&error];
    
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:@"Data" inManagedObjectContext:appDelegate.managedObjectContext]];
    
    NSArray *objectArray = [appDelegate.managedObjectContext executeFetchRequest:request error:&error];
    for (NSInteger i=0; i<records.count; i++)
        if (i == indexPath.row)
            [appDelegate.managedObjectContext deleteObject:[objectArray objectAtIndex:i]];
    
    [appDelegate.managedObjectContext save:&error];
}

- (IBAction)doExit {
    exit(0);
}

@end

