//
//  JSSRules.m
//  Pintoresco
//
//  Created by jsainz on 11/11/13.
//  Copyright (c) 2013 Javier Sainz. All rights reserved.
//

#define NSLog(FORMAT, ...) printf("%s\n", [[NSString stringWithFormat:FORMAT, ##__VA_ARGS__] UTF8String]);

#import "JSSRules.h"
#import "JSSUtils.h"

@implementation JSSRules

@synthesize params;

#pragma mark - Rule Class

+ (JSSRules*)rule {
    return [[JSSRules alloc] init];
}

- (id)init:(NSMutableArray*)p {
    self = [super init];
    if (self != nil)
        params = p;
    return self;
}

- (NSMutableArray*)delFeatureMember:(NSMutableArray*)item ind:(int)k par:(NSInteger)m cat:(NSString*)g code:(NSString*)pix {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    NSMutableArray *z = [[NSMutableArray alloc]init];
    NSMutableString *aux = [[NSMutableString alloc]init];
    NSString *str = [[NSString alloc]init];
    
    z = item;
    for (NSInteger i=0; i<z.count; i++) { // takes object under criterion
        iArray = [z objectAtIndex:i];
        aux = iArray.pictureCode;
        str = [aux substringWithRange:NSMakeRange(k, 1)];
        NSString *catCode = [NSString stringWithFormat:@"%c", (char)iArray.category];
        if ([catCode isEqualToString:g]) {
            if ([str isEqualToString:pix]) {
                if (m > 0) { // Criterion: 1; discarded any above 0
                    NSInteger q = m;
                    do {
                        iArray.category = 'N';
                        [z replaceObjectAtIndex:i withObject:iArray];
                        q--;
                    } while (q > 0); // Criterion: 0; discarded any above 0
                } // if (v1 > 0)
            } // else { // if ([str isEqualToString:@"1"])
        } // for (NSInteger i=0; i<z.count; i++)
    } // for (NSInteger i=0; i<z.count; i++) { // takes object under criterion
    return z;
} // (void)delMember:(NSMutableArray*)item index:(int)k param:(NSInteger)m

- (NSMutableArray*)delFeatureItem:(NSMutableArray*)item ind:(int)k {
    NSMutableArray *z = [[NSMutableArray alloc]init];
    BOOL ok = NO;
    
    z = item;
    NSInteger u0, v1, x0, y1;
    
    params = [self testFeatureRule:z index:k];
    k = [[params objectAtIndex:0] intValue]; // Char position
    u0 = [[params objectAtIndex:1] intValue];
    v1 = [[params objectAtIndex:2] intValue];
    x0 = [[params objectAtIndex:3] intValue];
    y1 = [[params objectAtIndex:4] intValue];
    // NSLog(@"K:%i U0:%ld V1:%ld X0:%ld Y1:%ld", k, (long)u0, (long)v1, (long)x0, (long)y1);
    
    // Filtering A-members which does satisfy the criterion but can be discarded
    if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))) {
        // NSLog(@"Category A");
        if (u0 > v1)  // 0s predominates for category A
            z = [self delFeatureMember:item ind:k par:v1 cat:@"A" code:@"1"];
        if (x0 < y1)  // 1s predominates for category B
            z = [self delFeatureMember:z ind:k par:x0 cat:@"B" code:@"0"];
        
        if (u0 < v1)  // 1s predominates for category A
            z = [self delFeatureMember:item ind:k par:u0 cat:@"A" code:@"0"];
        if (x0 > y1)  // 0s predominates for category B
            z = [self delFeatureMember:z ind:k par:y1 cat:@"B" code:@"1"];
        
        ok = YES;
    } // if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5)))
    
    // Filtering B-members which does satisfy the criterion but can be discarded
    if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))) {
        // NSLog(@"Category B");
        if (x0 > y1)  // 0s predominates for category B
            z = [self delFeatureMember:item ind:k par:y1 cat:@"B" code:@"1"];
        if (u0 < v1)  // 1s predominates for category A
            z = [self delFeatureMember:z ind:k par:u0 cat:@"A" code:@"0"];
        
        if (x0 < y1)   // 1s predominate for category B
            z = [self delFeatureMember:item ind:k par:x0 cat:@"B" code:@"0"];
        if (u0 > v1)  // 0s predominates for category A
            z = [self delFeatureMember:z ind:k par:v1 cat:@"A" code:@"1"];
        
        ok = YES;
    } // if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7)))
    return z;
} // (NSMutableArray*)delItems:(NSMutableArray*)item ind:(int)k

- (NSMutableArray*)testFeatureRule:(NSMutableArray*)item index:(int)k {
    BOOL OK = NO;
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    NSInteger u0 = 0; NSInteger v1 = 0; NSInteger x0 = 0; NSInteger y1 = 0;
    
    for (NSInteger i=0; i<item.count; i++){
        iArray = [item objectAtIndex:i];
        NSString *catCode = [NSString stringWithFormat:@"%c", (char)iArray.category];
        // aux = iArray.pictureCode;
        // str = [iArray.pictureCode substringWithRange:NSMakeRange(k, 1)];
        if ([[iArray.pictureCode substringWithRange:NSMakeRange(k, 1)] isEqualToString:@"0"]) {
            if ([catCode isEqualToString:@"A"])
                u0++;
            else // ([catCode isEqualToString:@"B"])
                x0++;
        } // if ([str isEqualToString:@"0"])
        else { // ([str isEqualToString:@"1"])
            if ([catCode isEqualToString:@"A"])
                v1++;
            else // ([catCode isEqualToString:@"B"])
                y1++;
        } // if ([str isEqualToString:@"1"])
        // NSLog(@"%c %@", iArray.category, iArray.pictureCode);
    } // for (NSInteger i=0; i<item.count; i++){
    
    if ((((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))) ||
        (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))))
        OK = YES;
    
    NSMutableArray *p = [[NSMutableArray alloc]init];
    [p addObject: [NSNumber numberWithInt:k]];
    [p addObject: [NSNumber numberWithInt:u0]];
    [p addObject: [NSNumber numberWithInt:v1]];
    [p addObject: [NSNumber numberWithInt:x0]];
    [p addObject: [NSNumber numberWithInt:y1]];
    return p;
} // (NSMutableArray*)testFeatureRule:(NSMutableArray*)item index:(int)k

// Creates an exemplar series classified by a unique feature
- (NSMutableArray*)featureRule:(NSMutableArray*)item {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    BOOL OK = NO;
    NSInteger k, u0, v1, x0, y1;
    do {
        JSSUtils *bin = [[JSSUtils alloc]init];
        
        while (!OK) {
            k = [bin assignNum:1 limit:4];
            if (!OK) {
                bin = [[JSSUtils alloc]init];
                item = [bin setCat:item aMembers:7 bMembers:5];
                params = [self testFeatureRule:item index:k];
                u0 = [[params objectAtIndex:1]intValue];
                v1 = [[params objectAtIndex:2]intValue];
                x0 = [[params objectAtIndex:3]intValue];
                y1 = [[params objectAtIndex:4]intValue];
                
                if ((((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))) ||
                    (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))))
                    OK = YES;
            } // if (!assign)
        } // while (!OK)
    } while (!OK);
    // NSLog(@"OK:%d K:%i A: U0:%ld V1:%ld B: X0:%ld Y1:%ld", OK, k, (long)u0, (long)v1, (long)x0, (long)y1);
    
    NSMutableArray *media = [[NSMutableArray alloc]init];
    media = [self delFeatureItem:item ind:k];
    media = [self checkFile:media];
    for (NSInteger i=0; i<media.count; i++) {
        iArray = [media objectAtIndex:i];
        NSLog(@"Feature Rule: %c %@", iArray.category, iArray.pictureCode);
    } // for (NSInteger i=0; i<item.count; i++)
    return media;
} // (void)featureRule:(NSMutableArray*)item

- (NSMutableArray*)delEqualItem:(NSMutableArray*)m item:(JSSStimulus*)r num:(NSInteger)n ind:(NSInteger)i {
    NSInteger q = n;
    do {
        r.category = 'N';
        [m replaceObjectAtIndex:i withObject:r];
        q--;
    } while (q > 0);
    return m;
} // (NSMutableArray*)delEquals:(NSMutableArray*)m item:(ItemArray*)r num:(NSInteger)n ind:(NSInteger)i

- (NSMutableArray*)testEqualRule:(NSMutableArray*)item {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    NSMutableArray *media = NSMutableArray.new;
    media = item;
    NSInteger u0 = 0, v1 = 0, x0 = 0, y1 = 0;
    for (NSInteger i=0; i<media.count; i++) {
        iArray = [media objectAtIndex:i];
        if ([iArray.test isEqualToString:@"u0"]) u0++;
        if ([iArray.test isEqualToString:@"v1"]) v1++;
        if ([iArray.test isEqualToString:@"x0"]) x0++;
        if ([iArray.test isEqualToString:@"y1"]) y1++;
        // NSLog(@"Test: %@ Cat:%c Code:%@", iArray.test, iArray.category, iArray.pictureCode);
    }
    // NSLog(@"U0:%ld V1:%ld B: X0:%ld Y1:%ld", (long)u0, (long)v1, (long)x0, (long)y1);
    
    for (NSInteger i=0; i<media.count; i++) { // takes object under criterion
        iArray = [media objectAtIndex:i];
        if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))){
            if ((u0 < v1) && ([iArray.test isEqualToString:@"u0"])) // Discard UO-exemplars
                media = [self delEqualItem:media item:iArray num:u0 ind:i];
            if ((u0 > v1) && ([iArray.test isEqualToString:@"v1"])) // Discard V1-exemplars
                media = [self delEqualItem:media item:iArray num:v1 ind:i];
            if ((x0 > y1) && ([iArray.test isEqualToString:@"y1"])) // Discard Y1-exemplars
                media = [self delEqualItem:media item:iArray num:y1 ind:i];
            if ((x0 < y1) && ([iArray.test isEqualToString:@"x0"])) // Discard X0-exemplars
                media = [self delEqualItem:media item:iArray num:x0 ind:i];
        } // if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5)))
        
        if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))){
            if ((u0 > v1) && ([iArray.test isEqualToString:@"v1"])) // Discard V1-exemplars
                media = [self delEqualItem:media item:iArray num:v1 ind:i];
            if ((u0 < v1) && ([iArray.test isEqualToString:@"u0"])) // Discard U0-exemplars
                media = [self delEqualItem:media item:iArray num:u0 ind:i];
            if ((x0 < y1) && ([iArray.test isEqualToString:@"x0"])) // Discard X0-exemplars
                media = [self delEqualItem:media item:iArray num:x0 ind:i];
            if ((x0 > y1) && ([iArray.test isEqualToString:@"y1"])) // Discard Y1-exemplars
                media = [self delEqualItem:media item:iArray num:y1 ind:i];
        } // if (((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5)))
    } // for (NSInteger i=0; i<media.count; i++)
    
    for (NSInteger i=0; i<media.count; i++) { // takes object under criterion
        iArray = [media objectAtIndex:i];
        if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))) {
            if ((x0 < y1) && ([iArray.test isEqualToString:@"x0"])) // Discard XO-exemplars
                media = [self delEqualItem:media item:iArray num:x0 ind:i];
            if ((x0 > y1) && ([iArray.test isEqualToString:@"y1"]))  // Discard Y1-exemplars
                media = [self delEqualItem:media item:iArray num:y1 ind:i];
            if ((u0 > v1) && ([iArray.test isEqualToString:@"v1"]))  // Discard V1-exemplars
                media = [self delEqualItem:media item:iArray num:v1 ind:i];
            if ((u0 < v1) && ([iArray.test isEqualToString:@"u0"]))  // Discard U0-exemplars
                media = [self delEqualItem:media item:iArray num:u0 ind:i];
        } // if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7)))
        
        if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))) {
            if ((x0 > y1) && ([iArray.test isEqualToString:@"y1"]))  // Discard Y1-exemplars
                media = [self delEqualItem:media item:iArray num:y1 ind:i];
            if ((x0 < y1) && ([iArray.test isEqualToString:@"x0"]))  // Discard X0-exemplars
                media = [self delEqualItem:media item:iArray num:x0 ind:i];
            if ((u0 > v1) && ([iArray.test isEqualToString:@"v1"])) // Discard V1-exemplars
                media = [self delEqualItem:media item:iArray num:v1 ind:i];
            if ((u0 < v1) && ([iArray.test isEqualToString:@"u0"]))  // Discard U0-exemplars
                media = [self delEqualItem:media item:iArray num:u0 ind:i];
        } // if (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7)))
    } // for (NSInteger i=0; i<media.count; i++)
    return media;
} // (NSMutableArray*)testEqualRule:(NSMutableArray*)item

// Creates an exemplar series classified by an unique emergent feature made of three primitives
- (NSMutableArray*)equalRule:(NSMutableArray*)item {
    NSString *fst = [[NSString alloc]init];
    NSString *snd = [[NSString alloc]init];
    NSString *trd = [[NSString alloc]init];
    
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    NSMutableArray *media = NSMutableArray.new;
    
    BOOL ok = NO;
    NSInteger u0 = 0, v1 = 0, x0 = 0, y1 = 0;
    do {
        JSSUtils *bin = [[JSSUtils alloc]init];
        bin = [[JSSUtils alloc]init];
        item = [bin setCat:item aMembers:7 bMembers:5];
        u0 = 0, v1 = 0, x0 = 0, y1 = 0;
        
        // NSLog(@"# A's:%d B's:%d", [bin numOfCat:item label:'A'], [bin numOfCat:item label:'B']);
        for (NSInteger i=0; i<item.count; i++){
            iArray = [item objectAtIndex:i];
            fst = [iArray.pictureCode substringWithRange:NSMakeRange(1, 1)];
            snd = [iArray.pictureCode substringWithRange:NSMakeRange(2, 1)];
            trd = [iArray.pictureCode substringWithRange:NSMakeRange(3, 1)];
            
            if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"A"]) {
                if ((fst = snd) && (snd = trd)) { // Equal Rule
                    if ((([fst isEqualToString:@"0"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"0"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"0"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"u0"]);
                        [media addObject:iArray];
                        u0++;
                    }
                    if ((([fst isEqualToString:@"1"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"1"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"1"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"v1"]);
                        [media addObject:iArray];
                        v1++;
                    }
                }
            } // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"A"])
            else { // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"B"])
                if ((fst = snd) && (snd = trd)) { // Equal Rule
                    if ((([fst isEqualToString:@"0"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"0"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"0"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"x0"]);
                        [media addObject:iArray];
                        x0++;
                    }
                    if ((([fst isEqualToString:@"1"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"1"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"1"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"y1"]);
                        [media addObject:iArray];
                        y1++;
                    }
                }
            } // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"B"]
        } // for (NSInteger i=0; i<item.count; i++)
        
        if ((((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))) ||
            (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))))
            ok = YES;
        else {
            [media removeAllObjects];
            media = NSMutableArray.new;
        }
    } while (!ok);
    // NSLog(@"OK:%d A: U0:%ld V1:%ld B: X0:%ld Y1:%ld", ok, (long)u0, (long)v1, (long)x0, (long)y1);
    
    media = [self testEqualRule:media];
    media = [self checkFile:media];
    for (NSInteger i=0; i<media.count; i++) {
        iArray = [media objectAtIndex:i];
        NSLog(@"Equal Rule: %c %@", iArray.category, iArray.pictureCode);
    } // for (NSInteger i=0; i<item.count; i++
    return media;
} // (void)equalRule:(NSMutableArray*)item

// Creates an exemplar series classified by an unique emergent feature made of three primitives
- (NSMutableArray*)unEqualRule:(NSMutableArray*)item {
    NSString *fst = [[NSString alloc]init];
    NSString *snd = [[NSString alloc]init];
    NSString *trd = [[NSString alloc]init];
    
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    NSMutableArray *media = NSMutableArray.new;
    
    BOOL ok = NO;
    NSInteger u0 = 0, v1 = 0, x0 = 0, y1 = 0;
    do {
        JSSUtils *bin = [[JSSUtils alloc]init];
        bin = [[JSSUtils alloc]init];
        item = [bin setCat:item aMembers:7 bMembers:5];
        u0 = 0, v1 = 0, x0 = 0, y1 = 0;
        
        // NSLog(@"# A's:%d B's:%d", [bin numOfCat:item label:'A'], [bin numOfCat:item label:'B']);
        for (NSInteger i=0; i<item.count; i++){
            iArray = [item objectAtIndex:i];
            fst = [iArray.pictureCode substringWithRange:NSMakeRange(1, 1)];
            snd = [iArray.pictureCode substringWithRange:NSMakeRange(2, 1)];
            trd = [iArray.pictureCode substringWithRange:NSMakeRange(3, 1)];
            
            if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"A"]) {
                if ((fst = snd) && (snd != trd)) { // Equal Rule
                    if ((([fst isEqualToString:@"0"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"0"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"0"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"u0"]);
                        [media addObject:iArray];
                        u0++;
                    }
                    if ((([fst isEqualToString:@"1"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"1"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"1"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"v1"]);
                        [media addObject:iArray];
                        v1++;
                    }
                }
            } // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"A"])
            else { // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"B"])
                if ((fst = snd) && (snd != trd)) { // Equal Rule
                    if ((([fst isEqualToString:@"0"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"0"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"0"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"x0"]);
                        [media addObject:iArray];
                        x0++;
                    }
                    if ((([fst isEqualToString:@"1"]) && ([fst isEqualToString:snd])) ||
                        (([fst isEqualToString:@"1"]) && ([fst isEqualToString:trd])) ||
                        (([snd isEqualToString:@"1"]) && ([snd isEqualToString:trd]))) {
                        iArray.test = ([NSString stringWithFormat:@"%@",@"y1"]);
                        [media addObject:iArray];
                        y1++;
                    }
                }
            } // if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"B"]
        } // for (NSInteger i=0; i<item.count; i++)
        
        if ((((u0 >= 7) || (v1 >= 7)) && ((x0 >= 5) || (y1 >= 5))) ||
            (((u0 >= 5) || (v1 >= 5)) && ((x0 >= 7) || (y1 >= 7))))
            ok = YES;
        else {
            [media removeAllObjects];
            media = NSMutableArray.new;
        }
    } while (!ok);
    // NSLog(@"OK:%d A: U0:%ld V1:%ld B: X0:%ld Y1:%ld", ok, (long)u0, (long)v1, (long)x0, (long)y1);
    
    media = [self testEqualRule:media]; // The same test is used for Unequal Rules
    media = [self checkFile:media];
    for (NSInteger i=0; i<media.count; i++) {
        iArray = [media objectAtIndex:i];
        NSLog(@"Unequal Rule: %c %@", iArray.category, iArray.pictureCode);
    } // for (NSInteger i=0; i<media.count; i++)
    return media;
} // (void)unEqualRule:(NSMutableArray*)item

- (NSMutableArray*)exemplarRule:(NSMutableArray*)media {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    JSSUtils *bin = [[JSSUtils alloc]init];
    media = [bin setCat:media aMembers:7 bMembers:5];
    media = [self checkFile:media];
    for (NSInteger i=0; i<media.count; i++) {
        iArray = [media objectAtIndex:i];
        NSLog(@"Exemplar Rule: %c %@", iArray.category, iArray.pictureCode);
    } // for (NSInteger i=0; i<media.count; i++)
    return media;
} // (void)exemplarRule:(NSMutableArray*)item

- (NSMutableArray*)discardItem:(NSMutableArray*)item code:(char)c {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    JSSUtils *bin = [[JSSUtils alloc]init];
    int k = [bin assignNum:0 limit:15];
    iArray = [item objectAtIndex:k];
    NSString *catCode = [NSString stringWithFormat:@"%c",(char)iArray.category];
    if ([catCode isEqualToString:[NSString stringWithFormat:@"%c", c]]) {
        iArray.category = 'N';
        [item replaceObjectAtIndex:k withObject:iArray];
    }
    return item;
} // (NSMutableArray*)discardItem:(NSMutableArray*)item code:(char)c

- (NSMutableArray*)checkFile:(NSMutableArray*)item {
    JSSStimulus *iArray = [[JSSStimulus alloc]init];
    JSSUtils *bin = [[JSSUtils alloc]init];
    NSInteger a = 0, b = 0;
    for (NSInteger i=0; i<item.count; i++) {
        iArray = [item objectAtIndex:i];
        if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"A"]) a++;
        else if ([[NSString stringWithFormat:@"%c", (char)iArray.category] isEqualToString:@"B"]) b++;
        // NSLog(@"Exemplar Rule: %c %@", iArray.category, iArray.pictureCode);
    }
    if (a >= b) {
        while ([bin numOfCat:item label:'A'] > 7)
            item = [self discardItem:item code:'A'];
        while ([bin numOfCat:item label:'B'] > 5)
            item = [self discardItem:item code:'B'];
    }
    else if (a < b) {
        while ([bin numOfCat:item label:'A'] > 5)
            item = [self discardItem:item code:'A'];
        while ([bin numOfCat:item label:'B'] > 7)
            item = [self discardItem:item code:'B'];
    }
    return item;
} // (NSMutableArray*)checkFile:(NSMutableArray*)item

@end
